﻿using FlowChartModule.Model;
using FlowChartModule.Views;
using Newtonsoft.Json;
using ParentService.ParentService;
using Prism.Ioc;
using Prism.Regions;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Shapes;
using Prism.Mvvm;
using System.Xml.Linq;
using CommonUtils.运行信号量;

namespace FlowChartModule.绘制模块
{
    public class DrawModule : BindableBase
    {
        private List<Element> elements=new List<Element>();
        private Element _CurrentSelectElement;//当前列表鼠标按下的元素

        private Element _CurrentClickElement;//当前列表鼠标点击的元素

        private Element _CurrentFlowSelectElement;//当前流程鼠标按下的元素

        private Element _CurrentFlowClickElement;//当前流程鼠标点击的元素

        private Element _CurrentMouseEnterElement;//当前流程鼠标悬停的元素

        private  string _CurrentStatus ="";//当前操作状态

        private int _CurrentIndex=0;//当前运行元素Index

        private Connection_Line _connection_Line;//当前画线

        private List<Element> CurrentFlowElements=new List<Element>();

        public bool  isMouseDown=false;

        private Canvas _canvas;
        public Canvas Canvas { get; set; }
        public Element CurrentSelectElement
        {
            get
            {
                return _CurrentSelectElement;

            }
            set
            {
                _CurrentSelectElement = value;
            }
        }

        public Element CurrentClickElement
        {
            get
            {
                return _CurrentClickElement;

            }
            set
            {
                _CurrentClickElement = value;
                _CurrentClickElement.RequestNavigate(_regionManager);
            }
        }

        public Element CurrentFlowClickElement { get => _CurrentFlowClickElement; set => _CurrentFlowClickElement = value; }
        public Connection_Line Connection_Line { get => _connection_Line; set => _connection_Line = value; }
        public List<Element> Elements { get => elements; set => elements = value; }
        public int CurrentIndex { get => _CurrentIndex; set { CurrentStatus = value.ToString(); SetProperty(ref _CurrentIndex, value); } }
        public string CurrentStatus { get => _CurrentStatus; set { SetProperty(ref _CurrentStatus, value); } }

        private IRegionManager _regionManager;
        private IContainerProvider _container;
        private AppData appData=new AppData();
        private Point MouseDownPoint;
        private Point MouseMovePoint;
        private Point CurrentPoint=new Point();
        private Element CurrentStartElement=null;


        private  List<Connection_Line>  LinePoint=new List<Connection_Line>();
        public DrawModule(IRegionManager regionManager, IContainerProvider container)
        {
            this._regionManager = regionManager;
            this._container = container;
        }

        public void DrawRectangle(Point point)
        {
            if ( CurrentSelectElement != null )
            {

                if ( CurrentFlowElements.Find(x => x.Id == CurrentSelectElement.Id) == null )
                {
                    Canvas.SetLeft(CurrentSelectElement.rect, point.X - CurrentSelectElement.rect.Width / 2);
                    Canvas.SetTop(CurrentSelectElement.rect, point.Y - CurrentSelectElement.rect.Height / 2);

                    CurrentSelectElement.rect.MouseDown += Rect_MouseDown;
                    CurrentSelectElement.rect.MouseMove += Rect_MouseMove;
                    CurrentSelectElement.rect.MouseUp += Rect_MouseUp; ;
                    CurrentSelectElement.rect.MouseEnter += Rect_MouseEnter;
                    CurrentSelectElement.rect.MouseLeave += Rect_MouseLeave;
                    CurrentSelectElement.rect.MouseRightButtonDown += Rect_MouseRightButtonDown;




                    if ( CurrentSelectElement.parentClass is StartService )
                    {
                        if ( CurrentStartElement == null )
                        {
                            CurrentStartElement = CurrentSelectElement;
                            Canvas.Children.Add(CurrentSelectElement.rect);
                            CurrentFlowElements.Add(CurrentSelectElement);
                        }
                    }
                    else
                    {

                        Canvas.Children.Add(CurrentSelectElement.rect);
                        CurrentFlowElements.Add(CurrentSelectElement);
                    }

                }

            }
            _CurrentFlowSelectElement = null;
            CurrentSelectElement = null;
        }



        private void Rect_MouseRightButtonDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            _regionManager.Regions[ "EditRegion" ].RequestNavigate(_CurrentMouseEnterElement.SourcesName);
            ConcreteVisitor visitor = new ConcreteVisitor(); // 创建访问者实例
            visitor._container = _container;
            _CurrentMouseEnterElement.parentClass.Accept(visitor);
        }

        private void Rect_MouseLeave(object sender, System.Windows.Input.MouseEventArgs e)
        {
            ( ( Element ) ( ( Canvas ) sender ).Tag ).left.ellipse.Fill = Brushes.Transparent;
            ( ( Element ) ( ( Canvas ) sender ).Tag ).up.ellipse.Fill = Brushes.Transparent;
            ( ( Element ) ( ( Canvas ) sender ).Tag ).down.ellipse.Fill = Brushes.Transparent;
            ( ( Element ) ( ( Canvas ) sender ).Tag ).right.ellipse.Fill = Brushes.Transparent;
            _CurrentMouseEnterElement = null;
        }

        private void Rect_MouseEnter(object sender, System.Windows.Input.MouseEventArgs e)
        {
            _CurrentMouseEnterElement = ( Element ) ( ( Canvas ) sender ).Tag;
            _CurrentMouseEnterElement.left.ellipse.Fill = Brushes.Green;
            _CurrentMouseEnterElement.up.ellipse.Fill = Brushes.Green;
            _CurrentMouseEnterElement.down.ellipse.Fill = Brushes.Green;
            _CurrentMouseEnterElement.right.ellipse.Fill = Brushes.Green;


        }

        private void Rect_MouseUp(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            isMouseDown = false;
            if ( CurrentFlowClickElement != null && Connection_Point._isMouseDown && _CurrentMouseEnterElement != null )
            {
                Connection_Line.Endelement = _CurrentMouseEnterElement;
                ( ( Element ) ( ( Canvas ) sender ).Tag ).connection_Lines.Add(Connection_Line);
            }



            Connection_Point._isMouseDown = false;
            _CurrentFlowSelectElement = null;
            CurrentFlowClickElement = null;
            System.Windows.Input.Mouse.Capture(null);
        }

        private void Rect_MouseMove(object sender, System.Windows.Input.MouseEventArgs e)
        {
            if ( _CurrentFlowSelectElement != null )
            {
                MouseMovePoint = e.GetPosition(Canvas);
                double _X= CurrentPoint.X+ MouseMovePoint.X-MouseDownPoint.X;
                if ( _X < 0 ) _X = 0;
                double _Y= CurrentPoint.Y+ MouseMovePoint.Y-MouseDownPoint.Y;
                if ( _Y < 0 ) _Y = 0;
                _CurrentFlowSelectElement.rect.SetValue(Canvas.LeftProperty, _X);
                _CurrentFlowSelectElement.rect.SetValue(Canvas.TopProperty, _Y);
                for ( int i = 0; i < _CurrentFlowSelectElement.connection_Lines.Count; i++ )
                {
                    if ( _CurrentFlowSelectElement.connection_Lines[ i ].Startelement == _CurrentFlowSelectElement )
                    {
                        double x=LinePoint[i].curveModel.StartPoint.X+(MouseMovePoint.X - MouseDownPoint.X);
                        double y=LinePoint[i].curveModel.StartPoint.Y+(MouseMovePoint.Y - MouseDownPoint.Y);
                        _CurrentFlowSelectElement.connection_Lines[ i ].curveModel.StartPoint = new Point(x, y);
                        _CurrentFlowSelectElement.connection_Lines[ i ].DrewbezierRef(Canvas);
                    }
                    else
                    {
                        double x=LinePoint[i].curveModel.EndPoint.X+(MouseMovePoint.X - MouseDownPoint.X);
                        double y=LinePoint[i].curveModel.EndPoint.Y+(MouseMovePoint.Y - MouseDownPoint.Y);
                        _CurrentFlowSelectElement.connection_Lines[ i ].curveModel.EndPoint = new Point(x, y);
                        _CurrentFlowSelectElement.connection_Lines[ i ].DrewbezierRef(Canvas);

                    }
                }
            }
        }

        private void Rect_MouseDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            isMouseDown = true;
            MouseDownPoint = e.GetPosition(Canvas);

            CurrentFlowClickElement = ( Element ) ( ( Canvas ) sender ).Tag;
            if ( !Connection_Point._isMouseDown )
            {
                _CurrentFlowSelectElement = ( Element ) ( ( Canvas ) sender ).Tag;
                LinePoint = new List<Connection_Line>();
                LinePoint = JsonConvert.DeserializeObject<List<Connection_Line>>(JsonConvert.SerializeObject(_CurrentFlowSelectElement.connection_Lines));
                CurrentPoint.X = Canvas.GetLeft(_CurrentFlowSelectElement.rect);
                if ( CurrentPoint.X < 0 ) CurrentPoint.X = 0;
                CurrentPoint.Y = Canvas.GetTop(_CurrentFlowSelectElement.rect);
                if ( CurrentPoint.Y < 0 ) CurrentPoint.Y = 0;
                System.Windows.Input.Mouse.Capture(sender as Canvas);
            }


        }

        public void SavePrograms()
        {
            appData.programDatas.Clear();
            for ( int i = 0; i < CurrentFlowElements.Count; i++ )
            {
                List<xmLine>  xmlLinePoint=new List<xmLine>();
                for ( int j = 0; j < CurrentFlowElements[ i ].connection_Lines.Count; j++ )
                {
                    xmlLinePoint.Add(new xmLine()
                    {
                        startPoint = CurrentFlowElements[ i ].connection_Lines[ j ].curveModel.StartPoint,
                        endPoint = CurrentFlowElements[ i ].connection_Lines[ j ].curveModel.EndPoint,
                        startGuid = CurrentFlowElements[ i ].connection_Lines[ j ].Startelement.Id,
                        endGuid = CurrentFlowElements[ i ].connection_Lines[ j ].Endelement.Id,
                    });
                }
                appData.programDatas.Add(new ProgramData()
                {
                    guid = CurrentFlowElements[ i ].Id,
                    name = CurrentFlowElements[ i ]._Name,
                    parentClass = CurrentFlowElements[ i ].parentClass,
                    LinePoint = xmlLinePoint,
                    point = new Point(( int ) Canvas.GetLeft(CurrentFlowElements[ i ].rect), ( int ) Canvas.GetTop(CurrentFlowElements[ i ].rect)),
                    sourcesName = CurrentFlowElements[ i ].SourcesName
                });

            }
            var settings = new JsonSerializerSettings
            {
                TypeNameHandling = TypeNameHandling.All
            };
            string json=JsonConvert.SerializeObject( appData, settings);
            string filePath = @"D:\配置文件\user.json";
            File.WriteAllText(filePath, json);
            CurrentStatus = "保存完成";
        }


        public void LoadPrograms()
        {
            CurrentFlowElements.Clear();
            Canvas.Children.Clear();
            string filePath = @"D:\配置文件\user.json";
            string json= File.ReadAllText(filePath);

            var settings = new JsonSerializerSettings
            {
                TypeNameHandling = TypeNameHandling.All
            };
            appData = JsonConvert.DeserializeObject<AppData>(json, settings);

            for ( int i = 0; i < appData.programDatas.Count; i++ )
            {
                Element   element=new Element(appData.programDatas[i].name,_container,true,false);
                GetConcreteVisitor visitor = new GetConcreteVisitor(); // 创建访问者实例
                element.parentClass = appData.programDatas[ i ].parentClass;

                
                ConcreteVisitor visitor2 = new ConcreteVisitor(); // 创建访问者实例
                visitor2._container = _container;
                element.parentClass.Accept(visitor2);

                element._Name = appData.programDatas[ i ].name;
                element.Id = appData.programDatas[ i ].guid;
                element.SourcesName = appData.programDatas[ i ].sourcesName;
                Canvas.SetLeft(element.rect, appData.programDatas[ i ].point.X);
                Canvas.SetTop(element.rect, appData.programDatas[ i ].point.Y);
                Canvas.Children.Add(element.rect);

                element.rect.MouseDown += Rect_MouseDown;
                element.rect.MouseMove += Rect_MouseMove;
                element.rect.MouseUp += Rect_MouseUp; ;
                element.rect.MouseEnter += Rect_MouseEnter;
                element.rect.MouseLeave += Rect_MouseLeave;
                element.rect.MouseRightButtonDown += Rect_MouseRightButtonDown;

                CurrentFlowElements.Add(element);
                if ( element.parentClass is StartService )
                {
                    CurrentStartElement = element;
                }
                _CurrentFlowSelectElement = null;
                CurrentSelectElement = null;
            }
            List< xmLine >  loadLineList=new List<xmLine>();
            for ( int i = 0; i < CurrentFlowElements.Count; i++ )
            {
                for ( int j = 0; j < appData.programDatas[ i ].LinePoint.Count; j++ )
                {

                    Connection_Line = new Connection_Line(Canvas);
                    Connection_Line.curveModel.StartPoint = appData.programDatas[ i ].LinePoint[ j ].startPoint;
                    Connection_Line.curveModel.EndPoint = appData.programDatas[ i ].LinePoint[ j ].endPoint;
                    Connection_Line.curveModel.StartPoint = appData.programDatas[ i ].LinePoint[ j ].startPoint;

                    if ( loadLineList.Find(x => x.startPoint == Connection_Line.curveModel.StartPoint && x.endPoint == Connection_Line.curveModel.EndPoint) == null )
                    {
                        loadLineList.Add(appData.programDatas[ i ].LinePoint[ j ]);
                    }
                }
            }

            bool sok=true;
            bool eok=true;
            for ( int j = 0; j < loadLineList.Count; j++ )
            {
                Connection_Line = new Connection_Line(Canvas);

                sok = true;
                eok = true;
                for ( int i = 0; i < CurrentFlowElements.Count; i++ )
                {

                    if ( CurrentFlowElements[ i ].Id == loadLineList[ j ].startGuid && sok )
                    {
                        Connection_Line.Startelement = CurrentFlowElements[ i ];
                        Connection_Line.curveModel.StartPoint = loadLineList[ j ].startPoint;
                        Connection_Line.curveModel.EndPoint = loadLineList[ j ].endPoint;
                        Connection_Line.curveModel.StartPoint = loadLineList[ j ].startPoint;
                        CurrentFlowElements[ i ].connection_Lines.Add(Connection_Line);
                        sok = false;
                    }

                    if ( CurrentFlowElements[ i ].Id == loadLineList[ j ].endGuid && eok )
                    {
                        Connection_Line.Endelement = CurrentFlowElements[ i ];
                        Connection_Line.curveModel.StartPoint = loadLineList[ j ].startPoint;
                        Connection_Line.curveModel.EndPoint = loadLineList[ j ].endPoint;
                        Connection_Line.curveModel.StartPoint = loadLineList[ j ].startPoint;
                        CurrentFlowElements[ i ].connection_Lines.Add(Connection_Line);
                        eok = false;
                    }
                    if ( !eok && !sok )
                    {
                        Connection_Line.Drewbezier(this.Canvas);

                        break;

                    }
                }

            }

            CurrentStatus = "加载完成";

        }

        public void ClearPrograms()
        {
            CurrentFlowElements.Clear();
            Canvas.Children.Clear();
            CurrentStatus = "清除完成";
        }

        public void RunPrograms()
        {
            if (StaticManualResetEvent.ManualisStopEvent.WaitOne(50) ) return;
            StaticManualResetEvent.ManualisStopEvent.Set();

            Thread RunProgramsThread=new Thread(()=>
            {
              
                while ( StaticManualResetEvent.ManualisStopEvent.WaitOne(50) )
                {
                    CurrentIndex=0;

                    for ( int i = 0; i < CurrentStartElement.connection_Lines.Count; i++ )
                    {
                        RunIVisitor visitor = new RunConcreteVisitor(); // 创建访问者实例
                        if(CurrentStartElement.connection_Lines[ 0 ].Endelement.parentClass.AcceptRun(visitor))
                        {
                            CurrentStatus = "停止";
                            StaticManualResetEvent.ManualisStopEvent.Reset();
                            return;
                        }
                        GetProgram(CurrentStartElement.connection_Lines[ 0 ].Endelement);

                    }
                }
                CurrentStatus = "停止";

            });

            RunProgramsThread.Name = "流程运行线程";
            RunProgramsThread.IsBackground = true;
            RunProgramsThread.Start();
            CurrentStatus = "运行中";
        }

        public void StopPrograms()
        {

            StaticManualResetEvent.ManualisStopEvent.Reset();

        }

        public void GetProgram(Element element)
        {

            for ( int i = 0; i < element.connection_Lines.Count; i++ )
            {
                CurrentIndex++;
                if ( !StaticManualResetEvent.ManualisStopEvent.WaitOne(50) ) return;
                if ( element.connection_Lines[ i ].Startelement == element )
                {
                    RunIVisitor visitor = new RunConcreteVisitor(); // 创建访问者实例
                    if (element.connection_Lines[i].Endelement.parentClass.AcceptRun(visitor)) 
                    {
                        StaticManualResetEvent.ManualisStopEvent.Reset();
                        return;
                    }

                }
                if ( element.connection_Lines[ i ].Endelement != element )
                    GetProgram(element.connection_Lines[ i ].Endelement);
            }



        }
    }
}
