﻿using LogicalElement.ViewModels;
using Newtonsoft.Json;
using ParentService.ParentService;
using Prism.Ioc;
using S7NetModule.ViewModels;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using TcpClientModule.ViewModels;


namespace ParentService.ParentService
{
    public class ParentClass
    {

        public bool isErr=false;
        public virtual void Accept(IVisitor visitor) {  }

        public virtual bool AcceptRun(RunIVisitor visitor) { return false; }

        public virtual ParentClass GetServices(IService visitor) { return null; }


    }
    public interface IVisitor 
    {
        
        /// <summary>
        /// s7客户端
        /// </summary>
        /// <param name="child"></param>
        void Visit(S7NetService child);
        /// <summary>
        /// s7读
        /// </summary>
        /// <param name="child"></param>
        void Visit(S7ReadService child);
        /// <summary>
        /// s7写
        /// </summary>
        /// <param name="child"></param>
        void Visit(S7WriteService child);

        /// <summary>
        /// 启动
        /// </summary>
        /// <param name="child"></param>
        void Visit(StartService child);



        /// <summary>
        /// Tcp客户端
        /// </summary>
        /// <param name="child"></param>
        void Visit(TcpClientService child);

        /// <summary>
        /// Tcp发送
        /// </summary>
        /// <param name="child"></param>
        void Visit(TcpSendService child);

        /// <summary>
        /// Tcp接收
        /// </summary>
        /// <param name="child"></param>
        void Visit(TcpRcvService child);


        /// <summary>
        /// 自定义
        /// </summary>
        /// <param name="child"></param>
        void Visit(CustomService child);

    }



    public interface RunIVisitor
    {
        bool RunVisit(S7ReadService child);
        bool RunVisit(S7WriteService child);

        bool RunVisit(S7NetService child);

        bool RunVisit(TcpClientService child);

        bool RunVisit(TcpSendService child);

        bool RunVisit(TcpRcvService child);

        bool RunVisit(CustomService child);
    }

    public interface IService
    {
      
        S7ReadService GetService(S7ReadService child, IContainerProvider container);
      
        S7WriteService GetService(S7WriteService child, IContainerProvider container);

        
        StartService GetService(StartService child, IContainerProvider container);

        S7NetService GetService(S7NetService child, IContainerProvider container);

        TcpClientService GetService(TcpClientService child, IContainerProvider container);

        TcpSendService GetService(TcpSendService child, IContainerProvider container);

        TcpRcvService GetService(TcpRcvService child, IContainerProvider container);

        CustomService GetService(CustomService child, IContainerProvider container);
    }


    public class ConcreteVisitor : IVisitor
    {
        [JsonIgnore]
        public IContainerProvider _container;
        public void Visit(S7ReadService child)
        {
            child._container = _container;
            child.S7_ReadView.DataContext = child.ReadViewModel;

        }

        public void Visit(S7WriteService child )
        {
            child._container = _container;
            child.S7_WriteView.DataContext = child.WriteViewModel;
        }
        public void Visit(StartService child)
        {
            
        }

        public void Visit(S7NetService child)
        {
            child._container = _container;
            child.S7_NetView.DataContext = child.S7ViewModel;
            
        }
        
        public void Visit(TcpClientService child)
        {
            child._container = _container;
            child.TcpClient_View.DataContext = child.Tcp_ClientViewModel;
        }
        
        public void Visit(TcpSendService child)
        {
            child._container = _container;
            child.TcpSend_View.DataContext = child.Tcp_ClientSendModel;
        }

        public void Visit(TcpRcvService child)
        {
            child._container = _container;
            TcpClientViewModel.ItemAdded -= child.Tcp_ClientRcvModel.TcpClientConfig_ItemAdded;
            TcpClientViewModel.ItemRemoved -= child.Tcp_ClientRcvModel.TcpClientConfig_ItemRemoved;
            TcpClientViewModel.ItemAdded += child.Tcp_ClientRcvModel.TcpClientConfig_ItemAdded;
            TcpClientViewModel.ItemRemoved += child.Tcp_ClientRcvModel.TcpClientConfig_ItemRemoved;
            child.TcpRcv_View.DataContext = child.Tcp_ClientRcvModel;
        }

        public void Visit(CustomService child)
        {
            child._container = _container;
            child.Custom_View.DataContext = child.Custom_View;
        }
    }

    public class RunConcreteVisitor : RunIVisitor
    {
        public bool RunVisit(S7ReadService child)
        {
            child.ReadViewModel.isErr = false;
            child.ReadViewModel.ReadS7ClientCommand();
            return child.ReadViewModel.isErr;
        }

        public bool RunVisit(S7WriteService child)
        {
            child.WriteViewModel.isErr = false;
            child.WriteViewModel.WriteS7ClientCommand();
            return child.WriteViewModel.isErr;
        }

        public bool RunVisit(S7NetService child)
        {
            child.S7ViewModel.isErr = false;
            child.S7ViewModel.OpenAllClientCommand();
            return child.S7ViewModel.isErr;
        }

        public bool RunVisit(TcpClientService child)
        {
            child.Tcp_ClientViewModel.isErr = false;
            child.Tcp_ClientViewModel.OpenAllTcpClientCommand();
            return child.Tcp_ClientViewModel.isErr;
        }

        public bool RunVisit(TcpSendService child)
        {
            child.Tcp_ClientSendModel.isErr = false;
            child.Tcp_ClientSendModel.TcpSendCommand();
            return child.Tcp_ClientSendModel.isErr;
        }
        public bool RunVisit(TcpRcvService child)
        {
            child.Tcp_ClientRcvModel.isErr = false;
            child.Tcp_ClientRcvModel.ReceiveThread();
            return child.Tcp_ClientRcvModel.isErr;
        }
        public bool RunVisit(CustomService child)
        {
            child.CustomViewModel.isErr = false;
            child.CustomViewModel.OpenStartData();
            return child.CustomViewModel.isErr;
        }
    }

    public class GetConcreteVisitor : IService
    {
        public S7ReadService GetService(S7ReadService child, IContainerProvider container)
        {
            return container.Resolve<S7ReadService>();
        }

        public S7WriteService GetService(S7WriteService child, IContainerProvider container)
        {
            return container.Resolve<S7WriteService>();
        }

        public StartService GetService(StartService child, IContainerProvider container)
        {
            return container.Resolve<StartService>();
        }

        public S7NetService GetService(S7NetService child, IContainerProvider container)
        {
            return container.Resolve<S7NetService>();
        }

        public TcpClientService GetService(TcpClientService child, IContainerProvider container)
        {
            return container.Resolve<TcpClientService>();
        }

        public TcpSendService GetService(TcpSendService child, IContainerProvider container)
        {
            return container.Resolve<TcpSendService>();
        }

        public TcpRcvService GetService(TcpRcvService child, IContainerProvider container)
        {
            return container.Resolve<TcpRcvService>();
        }

        public CustomService GetService(CustomService child, IContainerProvider container)
        {
            return container.Resolve<CustomService>();
        }
    }

   

}
